﻿// <reference path="../ScriptReference/jquery-1.7.1-vsdoc.js"/>
/// <reference path="Mediasite.Core.js"/>
/// <reference path="Mediasite.Player.Core.js"/>
/// <reference path="Mediasite.Player.Localization.js"/>
/*global Mediasite OnDSPlayStateChangeEvt : true, OnDSPositionChangeEvt : true, OnDSOpenStateChangeEvt : true, OnDSScriptCommandEvt : true, OnDSMediaErrorEvt : true */

Mediasite.namespace("MediaPlayer");
Mediasite.namespace("MediaPlayer.Views");

Mediasite.MediaPlayer.ModelHlsDotJs = function (viewOptions) {
    var LIVE_START_UNSAFE_WINDOW = 30 * 1000; // milliseconds

    var result = new Mediasite.MediaPlayer.ModelHtml5(viewOptions);
    var location = viewOptions.selectedVideoSource.Location;
    var logger = Mediasite.Logging.LoggerFactory("Media HLS.js" + ":" + viewOptions.streamType, { separator: "\n" });
    var video;
    var hls;
    var errorLogic;

    // Error logic instance lifetime must span multiple Hls instances in order 
    // to track errors that may trigger recreation of the video element.
    errorLogic = new Mediasite.MediaPlayer.HlsJsErrorLogic(logger);
    errorLogic.raiseErrorDelegate = function (message) { result.raiseMediaError(message); };


    function shouldEnableHLSjsLogging() {
        return Mediasite.GetQueryParam("logHLSjs", Mediasite.GetQueryParam.AsBool);
    }

    result.setVideoElementSourceFunction = function (element, mediaSource, mediaType) {

        video = element.get(0);
        if (hls) {
            hls.destroy();
        }
        hls = new Hls({ debug: shouldEnableHLSjsLogging() });
        errorLogic.attach(hls);

        hls.attachMedia(video);
        hls.loadSource(viewOptions.selectedVideoSource.Location);

    };

    result.trustLiveVideoElementPosition = true;

    result.liveStartUnsafeWindow = LIVE_START_UNSAFE_WINDOW;

    result.Stop = function () {
        video.pause();
        result.reportStateChanged(Mediasite.Player.MediaState.Stopped);
    };

    // TogglePlayPause is called during live, so Pause=Stop.
    // TODO: If HLSjs is used for on-demand playback someday, this gets more complicated.
    result.Pause = result.Stop;

    result.FinalizeLive = function () {
        result.Stop();
        if (hls) {
            hls.destroy();
        }
    };

    return result;
};

Mediasite.MediaPlayer.HlsJsErrorLogic = function (logger) {
    var self = this;
    var errorMemoryWindow = 1 * 60 * 1000; // ms
    var maxErrorsInWindow = 9;
    var fatalErrorMemoryWindow = 5 * 60 * 1000;
    var maxFatalErrorsInWindow = 2;

    var hls;
    var errorMemory = [];
    var fatalErrorMemory = [];


    function rememberError(errorString) {
        errorMemory.push({ when: (new Date()).getTime(), error: errorString });
    }

    function rememberFatalError(errorString) {
        fatalErrorMemory.push({ when: (new Date()).getTime(), error: errorString });
    }

    function tooManyErrors() {
        return getNumErrorsInWindow(errorMemory, errorMemoryWindow) > maxErrorsInWindow;
    }

    function tooManyFatalErrors() {
        return getNumErrorsInWindow(fatalErrorMemory, fatalErrorMemoryWindow) > maxFatalErrorsInWindow;
    }

    function getNumErrorsInWindow(memory, window) {
        // Remove errors outside window.
        var minTime = (new Date()).getTime() - window;
        trimErrorMemory(memory, minTime);

        // Return a count of the remaining errors.
        return memory.length;
    }

    function trimErrorMemory(memory, minTime) {
        while (memory.length > 0 && memory[0].when < minTime) {
            memory.shift();
        }
    }

    function makeErrorDataString(errorData) {
        return "HLS.JS ERROR -- fatal: " + errorData.fatal + " ; type: " + errorData.type + "; details: " + errorData.details + ";";
    }

    function raiseErrorEvent(errorData) {
        var message = '';
        var locMP = Mediasite.Player.Localization.MediaPlayer;

        if (errorData.type === Hls.ErrorTypes.NETWORK_ERROR) {
            message = locMP.AdaptiveStreamingNetworkError;
        } else if (errorData.type === Hls.ErrorTypes.MEDIA_ERROR) {
            message = locMP.AdaptiveStreamingMediaError;
        } else if (errorData.type === Hls.ErrorTypes.MUX_ERROR) {
            message = locMP.AdaptiveStreamingMuxError;
        } else { // OTHER_ERROR
            message = locMP.GeneralError;
        }

        self.raiseErrorDelegate(message);
    }

    function handleHLSjsError(event, data) {
        var errorString = makeErrorDataString(data);
        logger.Error(errorString);


        //rememberError(errorString);
        //if (tooManyErrors()) {
        //    logger.Error("Too many errors within " + errorMemoryWindow + "ms.");
        //    raiseErrorEvent(data);
        //    return;
        //}

        if (data.fatal) {
            rememberFatalError(errorString);
            if (tooManyFatalErrors()) {
                logger.Error("Too many FATAL errors within " + fatalErrorMemoryWindow + "ms.");
                raiseErrorEvent(data);
                return;
            }

            switch (data.type) {
                case Hls.ErrorTypes.NETWORK_ERROR:
                    logger.Error("Fatal network error encountered. Attempting to recover.");
                    hls.startLoad();
                    break;
                case Hls.ErrorTypes.MEDIA_ERROR:
                    logger.Error("Fatal media error encountered. Attempting to recover.");
                    hls.recoverMediaError();
                    break;
                default:
                    // Cannot recover. Raise the error event, even if first error.
                    raiseError(data);
                    break;
            }
        }
    }


    this.raiseErrorDelegate = function () { };

    this.attach = function (hlsObj) {
        hls = hlsObj;
        hls.on(Hls.Events.ERROR, handleHLSjsError);
    };
};